// include libraries for controlling servos and debouncing buttons
#include <Servo.h>
#include <Bounce2.h>

// declare objects to represent the two servos
Servo bigServo;
Servo littleServo;

// declare object to represent the button
Bounce button = Bounce();

// set global variables
int notePositions[] = {109,104,99,94,89,85,79,75};
int preHitDelay = 150;
int postHitDelay = 50;
int beaterRestAngle = 100;
int beaterHitAngle = 117;
unsigned int tempoDelay;
int tempo = 60; // bpm
int currentNote = 0;
int currentSequence = 0;
unsigned long nextChimeTime = 0;
boolean sequencePlaying = false;
boolean chimesActive = true;
int ledPin = 2;
int noteSequence[4][22] = {
  {3,5,4,0,-1,-1,3,4,5,3,-1,-1,5,3,4,0,-1,-1,0,4,5,3},
  {5,4,3,0,-1,-1},
  {3,5,4,0,-1,-1,3,4,5,0},
  {5,3,4,0,-1,-1,0,4,5,3,-1,-1,5,4,3,0,-1}
};
int sequenceLengths[4] = {22,6,10,17};
int startTimeHours = 16; // set to current hour before uploading
int startTimeMinutes = 59; // set to current minute before uploading
unsigned long sketchStartTime = 60UL * 60UL * startTimeHours + 60UL * startTimeMinutes;
unsigned long secondsSinceMidnight;

void setup() {
  bigServo.attach(8);
  littleServo.attach(9);
  bigServo.write(90);
  littleServo.write(beaterRestAngle);
  button.attach(3, INPUT_PULLUP);
  button.interval(25);
  tempoDelay = 60000 / (tempo * 2) - preHitDelay - postHitDelay;
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, chimesActive);
  unsigned long quarterHourSeconds = 15 * 60;
  unsigned long nextQuarterHour = sketchStartTime / quarterHourSeconds;
  currentSequence = (nextQuarterHour + 1) % 4;
  nextChimeTime = (nextQuarterHour + 1) * quarterHourSeconds;
}

void loop() {
  secondsSinceMidnight = sketchStartTime + millis() / 1000;
  updateButton();
  if(secondsSinceMidnight >= nextChimeTime) {
    sequencePlaying = true;
    currentNote = 0;
    nextChimeTime += 15 * 60; // add 15 minutes
  }
  if(sequencePlaying && chimesActive) {
    playNote(noteSequence[currentSequence][currentNote]);
    currentNote ++;
    if(currentNote == sequenceLengths[currentSequence]) {
      if(currentSequence == 0) {
        // play bongs
        int numBongs = (secondsSinceMidnight / 60 / 60) % 12;
        if(numBongs == 0) numBongs = 12;
        for(int i=0; i<numBongs && chimesActive; i++) {
          delayWhileUpdating(1500);
          playNote(0);
        }
      }
      sequencePlaying = false;
      currentSequence = (currentSequence+1) % 4;
    }
  }
}

void playNote(int thisNote) {
  if(thisNote >= 0 && thisNote <= 7) {
    // note is valid, play note
    bigServo.write(notePositions[thisNote]);
    delayWhileUpdating(preHitDelay);
    littleServo.write(beaterHitAngle);
    delayWhileUpdating(postHitDelay);
    littleServo.write(beaterRestAngle);    
  } else {
    // delay instead of playing note
    delayWhileUpdating(preHitDelay + postHitDelay);
  }
  delayWhileUpdating(tempoDelay);
}

void delayWhileUpdating(int delayTime) {
  for(int i=0; i<delayTime; i++) {
    updateButton();
    delay(1);
  }
}

void updateButton() {
  button.update();
  if(button.fell()) {
    chimesActive = !chimesActive;
    digitalWrite(ledPin, chimesActive);
  }
}
