// include libraries for controlling servos and debouncing buttons
#include <Servo.h>
#include <Bounce2.h>

// declare objects to represent the two servos
Servo bigServo;
Servo littleServo;

// declare object to represent the button
Bounce button = Bounce();

// set global variables
int notePositions[] = {109,104,99,94,89,85,79,75};
int preHitDelay = 150;
int postHitDelay = 50;
int beaterRestAngle = 100;
int beaterHitAngle = 117;
unsigned int tempoDelay;
int tempo = 120; // bpm
int currentNote = 0;
boolean sequencePlaying = false;
int ledPin = 2;
int noteSequence[] = {
  2,-1,2,-1,3,-1,4,-1,4,-1,3,-1,2,-1,1,-1,0,-1,0,-1,1,-1,2,-1,2,-1,-1,1,1,-1,-1,-1,
  2,-1,2,-1,3,-1,4,-1,4,-1,3,-1,2,-1,1,-1,0,-1,0,-1,1,-1,2,-1,1,-1,-1,0,0,-1,-1,-1,
  1,-1,1,-1,2,-1,0,-1,1,-1,2,3,2,-1,0,-1,1,-1,2,3,2,-1,1,-1,0,-1,1,-1,4,-1,2,-1,
  2,-1,2,-1,3,-1,4,-1,4,-1,3,-1,2,-1,1,-1,0,-1,0,-1,1,-1,2,-1,1,-1,-1,0,0,-1,-1,-1,
};

void setup() {
  bigServo.attach(8);
  littleServo.attach(9);
  bigServo.write(90);
  littleServo.write(beaterRestAngle);
  button.attach(3, INPUT_PULLUP);
  button.interval(25);
  tempoDelay = 60000 / (tempo * 2) - preHitDelay - postHitDelay;
  pinMode(ledPin, OUTPUT);
}

void loop() {
  updateButton();
  if(sequencePlaying) {
    playNote(noteSequence[currentNote]);
    currentNote ++;
    if(currentNote == 128) {
      sequencePlaying = false;
      digitalWrite(ledPin, false);
    }
  }
}

void playNote(int thisNote) {
  if(thisNote >= 0 && thisNote <= 7) {
    // note is valid, play note
    bigServo.write(notePositions[thisNote]);
    delayWhileUpdating(preHitDelay);
    littleServo.write(beaterHitAngle);
    delayWhileUpdating(postHitDelay);
    littleServo.write(beaterRestAngle);    
  } else {
    // delay instead of playing note
    delayWhileUpdating(preHitDelay + postHitDelay);
  }
  delayWhileUpdating(tempoDelay);
}

void delayWhileUpdating(int delayTime) {
  for(int i=0; i<delayTime; i++) {
    updateButton();
    delay(1);
  }
}

void updateButton() {
  button.update();
  if(button.fell()) {
    sequencePlaying = !sequencePlaying;
    if(sequencePlaying) currentNote = 0;
    digitalWrite(ledPin, sequencePlaying);
  }
}
