// include MIDI library
#include <MIDI.h>
MIDI_CREATE_DEFAULT_INSTANCE();

// include libraries for controlling servos and debouncing buttons
#include <Servo.h>
#include <Bounce2.h>

// declare objects to represent the two servos
Servo bigServo;
Servo littleServo;

// declare object to represent the button
Bounce button = Bounce();

// set global variables
int notePositions[] = {109,104,99,94,89,85,79,75};
int beaterRestAngle = 100;
int beaterHitAngle = 117;
int postHitDelay = 50;
int ledPin = 2;

void setup() {
  bigServo.attach(8);
  littleServo.attach(9);
  bigServo.write(90);
  littleServo.write(beaterRestAngle);
  pinMode(ledPin, OUTPUT);
  MIDI.setHandleNoteOn(moveToMidiNote);
  MIDI.setHandleNoteOff(hitBeater);
  MIDI.begin(MIDI_CHANNEL_OMNI);
}

void loop() {
  MIDI.read();
}

int moveToMidiNote(byte channel, byte midiNote, byte velocity) {
  int noteNum = getGlockNoteFromMidiNote(midiNote);
  if(noteNum >= 0) {
    digitalWrite(ledPin, HIGH);
    bigServo.write(notePositions[noteNum]);
  } else {
    digitalWrite(ledPin, HIGH);
    delay(50);
    digitalWrite(ledPin, LOW);
    delay(50);
  }
}

int hitBeater(byte channel, byte midiNote, byte velocity) {
  int noteNum = getGlockNoteFromMidiNote(midiNote);
  if(noteNum >= 0) {
    digitalWrite(ledPin, LOW);
    littleServo.write(beaterHitAngle);
    delay(postHitDelay);
    littleServo.write(beaterRestAngle);
  } else {
    digitalWrite(ledPin, HIGH);
    delay(50);
    digitalWrite(ledPin, LOW);
    delay(50);
  }
}

int getGlockNoteFromMidiNote(int midiNote) {
  switch(midiNote) {
    case 60:
    return 0;
    case 62:
    return 1;
    case 64:
    return 2;
    case 65:
    return 3;
    case 67:
    return 4;
    case 69:
    return 5;
    case 71:
    return 6;
    case 72:
    return 7;
  }
  return -1;
}
